import resource
import logging
import logging.config
import logging.handlers
from util import Subject

class LoggerExtensionPoint:
    def __getitem__(self, point):
        if '.' in point:
            raise ValueError("sub extensions are not supported")
            #TODO: could use .level to get/set level etc
        return logging.getLogger(point)

    def __setitem__(self, point, value):
        raise ValueError("not supported")

    def __delitem__(self, point):
        raise ValueError("not supported")

class RotatingBufferHandler(logging.handlers.BufferingHandler, Subject):
    """a logger that holds messages up to a given number in the
    memory. if the capacity is exeeded, the oldest messages
    are thrown away.
    also implement the observer patern, so that a viewer can
    efficiently grab the messages"""
    def __init__(self, *args, **kwargs):
        Subject.__init__(self)
        logging.handlers.BufferingHandler.__init__(self, *args, **kwargs)
    def emit(self, *args, **kwargs):
        logging.handlers.BufferingHandler.emit(self, *args, **kwargs)
        self.notify()
    def flush(self):
        del self.buffer[-self.capacity:]
        #~ self.notify()

#add to namespace so that th ini file can reference it
logging.RotatingBufferHandler = RotatingBufferHandler

def init(config_filename):
    logging.config.fileConfig(config_filename)
