#!/usr/bin/env python
from twisted.internet import wxreactor
wxreactor.install()
from twisted.internet import reactor

import os, sys, traceback
import logger
logger.init('logger_config.ini')

log = logger.logging.getLogger('application')

#install text mode exception handler
def exconsole(*args):
    msg = ''.join(traceback.format_exception(*args))
    log.fatal(msg)
sys.excepthook = exconsole              #install console exception handler

import wx

#XXX TEST DEBUG use normal frames instead of MDI -> Linux usability
if sys.platform == 'linux2':
    wx.MDIParentFrame = wx.Frame
    wx.MDIChildFrame = wx.Frame

import resource, pluginmanager, languages
from wxutils import getImage

def exgui(*args):
    msg = ''.join(traceback.format_exception(*args))
    log.fatal(msg)
    wx.MessageDialog(None, msg, sys.argv[0] + " - Internal error", wx.OK | wx.ICON_INFORMATION).ShowModal()



#Constants
ID_EXIT         = wx.NewId()
#~ ID_CLOSEALL     = wx.NewId()

class MainFrame(wx.Frame):
    """main window with menu, other windows are shown within this window"""
    title = "Raw data viewer"
    def __init__(self, app):
        wx.Frame.__init__(self, None, -1, self.title)
        self.app = app
        
        try:
            self.icon = wx.IconFromBitmap(getImage('icon'))
            self.SetIcon(self.icon)
        except KeyError:
            self.icon = None
        
        #toolbar
        #self.toolbar = self.CreateToolBar(wx.TB_HORIZONTAL|wx.NO_BORDER)
        #self.toolbar.Realize()
        
        # create a statusbar
        #~ self.statusbar = sb = self.CreateStatusBar(2)
        #~ sb.SetStatusWidths([-1, 150])
        self.statusbar = sb = self.CreateStatusBar(1)
        sb.SetStatusWidths([-1])
        
        self.menus = {}
        self.menus[_("&File")] = wx.Menu()
        self.menus[_("&File")].Append(ID_EXIT, _("E&xit"), _("Terminate the program"))
        
        self.mainmenu = wx.MenuBar()
        self.mainmenu.Append(self.menus[_("&File")], _("&File"))
        
        wx.EVT_MENU(self, ID_EXIT, self.OnMenuExit)
        #~ self.SetMenuBar(self.mainmenu)
        
        wx.EVT_CLOSE(self, self.OnCloseWindow)
        
        self.pluginmanager = pluginmanager.PluginManager()
        self.pluginmanager['system'] = pluginmanager.SystemExtensionPoint()
        self.pluginmanager['system.persistent'] = pluginmanager.ExtensionPoint()
        self.pluginmanager['system.persistent.filename'] = 'client.persistent'
        self.pluginmanager['log'] = logger.LoggerExtensionPoint()
        self.pluginmanager['gui'] = pluginmanager.ExtensionPoint()
        self.pluginmanager['gui.wx'] = pluginmanager.ExtensionPoint()
        self.pluginmanager['gui.client'] = pluginmanager.ExtensionPoint() # For all the network related stuff
        self.pluginmanager['nogui'] = pluginmanager.ExtensionPoint()
        self.pluginmanager['gui.wx.version'] = wx.VERSION
        self.pluginmanager['gui.handle'] = self
        self.pluginmanager['gui.icon'] = self.icon
        self.pluginmanager['gui.statusmessage'] = self.writeStatustext
        pluginmanager.BUILTIN_EXTENSIONS.extend(['gui', 'gui.wx', 'gui.icon', 'gui.handle', 'gui.statusmessage'])
        #~ menu={'file': (_('E&xit'), ID_EXIT)}
        self.pluginmanager.loadPlugins(os.path.join(resource.getPluginPath(), 'nogui'))
        self.pluginmanager.loadPlugins(os.path.join(resource.getPluginPath(), 'wx'))
        #~ try:
            #~ self.pluginmanager.loadPlugins(os.path.join(resource.getHomePluginPath(), 'wx'))
        #~ except resource.NoSuchResourceError:
            #~ sys.stderr.write("no plugin directory in user home directory\n")
        self.pluginmanager.start()
        #now try to get the settings
        try:
            (x, y, w, h) = self.pluginmanager['persistent.mainwindow_PosSize']
        except:  #ignore any error, if the settings cannot be retreived
            log.exception('Could not load window size and position')
            self.SetSize((400, 200))
        else:
            self.SetDimensions(x, y, w, h)

        log.info('Ready')

    def writeStatustext(self, message):
        self.statusbar.SetStatusText(message, 0)

    def OnMenuExit(self, event):
        self.Close(1)

    def OnCloseWindow(self, event):
        log.info('Shutting down')
        #stop everything, save settings
        x,y = self.GetPositionTuple()
        w,h = self.GetSizeTuple()
        try:
            self.pluginmanager['persistent.mainwindow_PosSize'] = (x, y, w, h)
        except:
            log.exception('Could not save window size and position')
        
        #shutdown plugins
        self.pluginmanager.destroy()
        
        #destroy window
        #~ win = wxWindow_FindFocus()
        #~ if win != None:
            #~ win.Disconnect(-1, -1, wxEVT_KILL_FOCUS)
        self.Show(0)                    #hide window immediately
        self.app.doClose()              #application clean up
        self.Destroy()
        reactor.stop()

#----------------------------------------------------------------------

class App(wx.App):
    def OnInit(self):
        self.frame = MainFrame(self)
        self.frame.Show(True)
        self.SetTopWindow(self.frame)
        
        if wx.VERSION < (2, 3, 3):
             wx.MessageBox("You're using an outdated version of wxPython. Please upgrade",
                          'Version warning',
                          wx.OK | wx.ICON_WARNING)
        return True

    def doClose(self):
        pass

def main():
    #~ print sys.path
    app = App(0)
    sys.excepthook = exgui              #install GUI exception handler
    reactor.registerWxApp(app)
    reactor.run(0)
    sys.excepthook = exconsole          #reinstall console exception handler
    
if __name__=='__main__':
    main()
