"""
get filenames, files by name.

avoids problems with paths and the current directory etc.
"""

NAME='blockpuzzle'

import os

#try to find the modules directory
try:
    root = os.path.abspath(os.path.dirname(__file__))
except NameError:       #will happen if this module is started
    root = os.path.abspath(os.curdir)

#user home directory
try:
    home = os.path.join(os.environ['HOMEDRIVE'], os.environ['HOMEPATH']) #win32
except KeyError:
    try:
        home = os.environ['HOME'] #unix
    except KeyError:
        home = '.'

homedir = os.path.join(home, NAME)      #program home directory
#check if it exists, create it if not
#~ if not os.path.exists(homedir):
    #~ os.mkdir(homedir)
    #~ os.mkdir(os.path.join(homedir, 'plugins'))
    #~ os.mkdir(os.path.join(homedir, 'plugins', 'data'))
    #~ os.mkdir(os.path.join(homedir, 'plugins', 'nogui'))
    #~ os.mkdir(os.path.join(homedir, 'plugins', 'wx'))

# --- retreive data ---

class NoSuchResourceError(Exception): pass

def getProgramPath(*args):
    """calculate filename to plugins directory"""
    return os.path.abspath(os.path.join(root, '..', *args))

def getHomePath(*args):
    """calculate filename to plugins directory"""
    return os.path.abspath(os.path.join(homedir, *args))

def getPluginPath(*args):
    """calculate filename to plugins directory"""
    return getProgramPath('plugins', *args)

def getHomePluginPath(*args):
    """calculate filename to plugins directory"""
    path = getHomePath('plugins', *args)
    if os.path.exists(path):
        return path
    else:
        raise NoSuchResourceError

def getHelpFilePath(*args):
    """calculate filename to plugins/data"""
    return getProgramPath('help', *args)

def getFilePath(name):
    """calculate filename to plugins/data"""
    return getPluginPath('data', name)

def getHomeFilePath(*args):
    """calculate filename to plugins/data"""
    return getHomePluginPath('data', *args)

def get(name, mode='r'):
    """get a file object with the requested resource"""
    filename = getFilePath(name)
    if os.path.exists(filename):
        return file(filename, mode)
    raise KeyError("no resource with name %r found" % name)
