# -- bitmap handling --
#
import wx
import os
import resource

wx.InitAllImageHandlers()

imagelist = {}
FILETYPES = [
    ('.bmp', wx.BITMAP_TYPE_BMP),
    ('.png', wx.BITMAP_TYPE_PNG),
    ('.jpg', wx.BITMAP_TYPE_JPEG),
]

def _probeImage(pathfunction, name):
    for ext, type_id in FILETYPES:
        try:
            filename = pathfunction(name + ext)
        except resource.NoSuchResourceError:
            return None
        else:
            if os.path.exists(filename):
                print "load", filename
                imagelist[name] = wx.Bitmap(filename, type_id)
                return 1

def getImage(name):
    """get image from cache
    if image is not yet in the cache, search plugin data in home and 
    then plugin data in the program directory"""
    if name not in imagelist:
        if not _probeImage(resource.getHomeFilePath, name):
            _probeImage(resource.getFilePath, name)
    return imagelist[name]


builtin_translation = {
    'bookmark':         wx.ART_ADD_BOOKMARK,
    'del_bookmark':     wx.ART_DEL_BOOKMARK,
    'help_side_panel':  wx.ART_HELP_SIDE_PANEL,
    'help_settings':    wx.ART_HELP_SETTINGS,
    'help_book':        wx.ART_HELP_BOOK,
    'help_folder':      wx.ART_HELP_FOLDER,
    'help_page':        wx.ART_HELP_PAGE,
    'go_back':          wx.ART_GO_BACK,
    'go_forward':       wx.ART_GO_FORWARD,
    'go_up':            wx.ART_GO_UP,
    'go_down':          wx.ART_GO_DOWN,
    'go_to_parent':     wx.ART_GO_TO_PARENT,
    'go_home':          wx.ART_GO_HOME,
    'file_open':        wx.ART_FILE_OPEN,
    'print':            wx.ART_PRINT,
    'help':             wx.ART_HELP,
    'tip':              wx.ART_TIP,
    'report_view':      wx.ART_REPORT_VIEW,
    'list_view':        wx.ART_LIST_VIEW,
    'new_dir':          wx.ART_NEW_DIR,
    'folder':           wx.ART_FOLDER,
    'go_dir_up':        wx.ART_GO_DIR_UP,
    'executable_file':  wx.ART_EXECUTABLE_FILE,
    'normal_file':      wx.ART_NORMAL_FILE,
    'tick_mark':        wx.ART_TICK_MARK,
    'cross_mark':       wx.ART_CROSS_MARK,
    'error':            wx.ART_ERROR,
    'question':         wx.ART_QUESTION,
    'warning':          wx.ART_WARNING,
    'information':      wx.ART_INFORMATION,
}

def getBuiltinImage(name, size=(16,16)):
    #~ if inline_images.has_key(name):
        #~ bmp = inline_images[name]
        #~ if (bmp.GetWidth, bmp.GetHeight()) != size:
            #~ bmp = wx.ImageFromBitmap(bmp).Scale(*size).ConvertToBitmap()
    #~ else:
    if 1:       #inlide diabled because of compatibility probs with wx
        if hasattr(wx, 'ArtProvider_GetBitmap'):          #only available in newer versions
            bmp = wx.ArtProvider_GetBitmap(builtin_translation[name], wx.ART_TOOLBAR, size)
            if not bmp.Ok():
                bmp = wx.EmptyBitmap(*size)
        else:
            bmp = wx.EmptyBitmap(*size)
    return bmp

