import time, threading
from twisted.spread import pb

max_buffer_size = 5
max_wait = 1.0

class pbTransporter:
    def __init__(self, remote):
        self.remote = remote
        self.data = []
        self.parents = []
        self.last_send = time.time()
        self.semaphore = threading.Semaphore()

    def send_data(self):
        try:
            deferred = self.remote.callRemote('receive', self.data)
            deferred.addErrback(self.handleFailure)
            deferred.addCallback(self.arrived)
        except pb.DeadReferenceError:
            # The client has died - so why should I care ;-)
            pass

    def arrived(self, ret):
        self.data = []
        self.semaphore.release()

    def handleFailure(self, reason):
        # The client exited and failed to wait for the last messages in the 
        # queue - this is nothing serious
        print 'Failed to deliver data'
        
    def receive(self, data):
        self.data.append([data.time, data.source, data.value])
        if (len(self.data) > max_buffer_size or time.time() - self.last_send > max_wait) and self.semaphore.acquire(0):
            self.send_data()
            self.last_send = time.time()

### Plugin interface ###

depends = []
provides = ['nogui.transporter']

def init(manager):
    manager['nogui.transporter'] = pbTransporter
