from twisted.spread import pb
from twisted.internet import reactor
from twisted.cred import portal, checkers
import ConfigParser, traceback, sys, socket

manager = None

class Session(pb.Avatar):
    def __init__(self, name):
        self.filters = {}
        self.persistent = False
        self.avatarID = name
        self.hooks = {}
        self.transporter = None

    def perspective_endSession(self):
        # Remove all the hooks first
        for key in self.hooks.keys():
            hook = self.hooks[key]
            for x in hook.parents:
                x[0].removeChild(x[1], hook.receive)
            del self.hooks[key]

        # Now remove all the filters
        for key in self.filters:
            filter = self.filters[key]
            for x in filter.parents:
                x[0].removeChild(x[1], x[2])
            del self.filters[key]
                

    def perspective_removeHook(self, name, channel):
        try:
            hook = self.hooks[name + '.' + channel]
        except NameError:
            return  
        
        if name in core.sources.keys():
            inst = core.sources[name]
        elif name in self.filters.keys():
            inst = filters[name]

        inst.removeChild(channel, hook.receive)
        del self.hooks[name + '.' + channel]
        

    def perspective_addHook(self, name, channel, ref):
        if name in core.sources.keys() and channel in core.sources[name].output_channels:
            inst = core.sources[name]
        elif name in self.filters.keys() and channel in filters[name].output_channels:
            inst = filters[name]
        else:
            raise pb.Error("Name/Channel combination is invalid")

        if self.transporter is None:
            self.transporter = manager['nogui.transporter'](ref)

        inst.appendChild(channel, self.transporter.receive)

    def perspective_name(self):
        return core.name()

    # Returns a list of all the available data sources
    def perspective_getSources(self):
        return core.ListSources()
                        

    # Returns a list of all the available filter plugins. Includes output-plugins
    def perspective_getFilters(self):
        return core.ListFilters()

    def perspective_setFilterConfig(self, config, routing):
        # For filter in config: init
        for filter in config.keys():
            try:
                if not self.filters.has_key(filter):
                    self.filters[filter] = manager[config[filter][0] + '.inst']('name', config[filter][1])
            except Exception, e:
                return 'Error: %s' % str(e)

        # For filter in config: initialize
        map(lambda x: self.filters[x].initialize(), config.keys())
        
        # For filter in config: route
        for route in routing:
            self.filters[route[0]].appendChild(route[1], self.filters[route[2]].input_channels[route[3]])

    def perspective_getFilterConfig(self):
        result = {}
        for x in self.filters.keys():
            result[x] = self.filters[x].settings
            
        # TODO: Read out routing

        return result


class Realm:
    __implements__ = portal.IRealm

    def __init__(self, adminIface):
        self.avatars = {}
        self.adminIface = adminIface
        if core.ReadSrcConfig():
            try:
                self.restoreSessions()
            except Exception, e:
                print str(e)
                pass

    def restoreSessions(self):
        pass

    def cleanup(self, a):
        if a.persistent == False:
            a.perspective_endSession()
            try:
                del self.avatars[a.avatarID]
            except:
                pass

    def isInLimitedMode():
        return core.limitedMode

    def requestAvatar(self, avatarID, mind, *interfaces):
        assert pb.IPerspective in interfaces
        if avatarID == 'admin':
            return pb.IPerspective, self.adminIface, lambda:None
        elif self.avatars.has_key(avatarID):
            #Restore a session that was discontinued
            avatar = self.avatars[avatarID]
        else:
            avatar = Session(avatarID)
            self.avatars[avatarID] = avatar

        return pb.IPerspective, avatar, lambda a=avatar: self.cleanup(a)


#### Plugin-interface

provides = ['server.avatar']
depends = ['server.checker', 'server.adminiface', 'nogui.transporter', 'server.core']

def init(pluginmanager):
    print pluginmanager['server.interface']
    if pluginmanager['server.interface'] == 'avatar':
        global core; core = pluginmanager['server.core']
        global manager; manager = pluginmanager
        p = portal.Portal(Realm(manager['server.adminiface']()))
        p.registerChecker(manager['server.checker'](admin='admin'))
        reactor.listenTCP(8801, pb.PBServerFactory(p))
        manager['server.inst'] = p # No idea why I should need this
    
