import wx
import wx.py.version
import matplotlib
from pylab import *
matplotlib.use('WXAgg')
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
import matplotlib.numerix as nx
from twisted.spread import pb

class MonitorView:
    def __init__(self, gui):
        self.Panel = wx.Panel(gui, -1, style=wx.SUNKEN_BORDER)
        self.parent = gui
        self.callid = None

        self.figure = Figure()
        self.axes = self.figure.add_subplot(111)
        self.axes.set_title('Press a to add data')

        self.canvas = FigureCanvas(self.Panel, 1001, self.figure)
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        
        self.sizer.Add(self.canvas, 1, wx.LEFT | wx.TOP | wx.GROW)
        self.Panel.SetSizer(self.sizer)
        self.Panel.Fit()
        self.data = {}
        self.data['sinus'] = [Line2D([],[]), True]
        self.axes.add_line(self.data['sinus'][0])
        
        #self.canvas.mpl_connect('key_press_event', self.add_data)

    def add_data(self, event):
        self.receive([[nx.mlab.rand(1), 'sinus', nx.mlab.rand(1)]])
        
    def receive(self, data):
        try:
            do_plot = False
            for x in data:
                do_plot = do_plot or self.data[x[1]][1]
                line = self.data[x[1]][0]
                xdata = line.get_xdata()
                ydata = line.get_ydata()
            
                xdata.append(x[0])
                ydata.append(x[2])
                
                self.data[x[1]][0].set_data(xdata, ydata)
                
            if do_plot:
                self.figure.canvas.draw()
                self.canvas.gui_repaint()
        except KeyError:
            # If we deselect a source, a call might still be progressing.
            # Thus - we ignore this error
            pass

    def add_success(self, ret):
        return

    def remove_success(self, ret, source):
        del self.data[source]

    def add_failure(self, ret):
        print 'Failed to add handler'

    def set_scroll(self, amount):
        self.add_data(None)
        setp(self.figure.gca(), 'xlim', [0, amount])
        self.figure.canvas.draw()

    def EnableSource(self, source):
        self.data[source][1] = True
        self.axes.plot([0,1], [0,1])
        self.axes.add_line(self.data[source][0])
        self.figure.canvas.draw()
        self.canvas.gui_repaint()

    def DisableSource(self, source):
        self.data[source][1] = False
        self.axes.lines.remove(self.data[source][0])
        self.figure.canvas.draw()

    def AddSource(self, source):
        obj, channel = source.split('.')
        deferred = self.root.rootObject.callRemote('addHook', obj, channel, self.receiver)
        deferred.addCallbacks(self.add_success, self.add_failure)
        self.data[source] = [Line2D([],[]), False]

    def RemoveSource(self, source):
        obj, channel = source.split('.')
        deferred = self.root.rootObject.callRemote('removeHook', obj, channel)
        deferred.addCallback(self.remove_success, source)
    
#### Plugin interface ####
depends = ['gui', 'gui.menu', 'persistent']
provides = ['gui.plot']

def init(manager):
    manager['gui.plot'] = MonitorView(manager['gui.handle'])
