"""
This module provides some utility functions for wxPython
"""

import wx
from modules.wxutils import getImage, getBuiltinImage
from modules import pluginmanager

def messageBox(message, title = "Plugin Message"):
    dlg = wx.MessageDialog(g_manager['gui.handle'],
                             message,                                #message
                             title,                       #title
                             wx.OK | wx.ICON_INFORMATION        #button and icon
    )
    dlg.ShowModal()
    dlg.Destroy()

def askText(title='Enter something:', default=''):
    cdlg = wx.TextEntryDialog(g_manager['gui.handle'],
                                title,                      #desc
                                'Enter Text',               #window title
                                default,                    #default text
    )
    try:
        if cdlg.ShowModal() == wx.ID_OK:
            return cdlg.GetValue()
        else:
            raise ValueError("Aborted")
    finally:
        cdlg.Destroy()

def writeStatustext(message):
    g_manager['gui.handle'].statusbar.SetStatusText(message, 0)

# - - - Plugin Interface - - -
depends = ['gui']
provides = ['utility']

def action(manager):
    ogl.OGLInitialize()
    a = OGLView(manager.gui)
    a.Show()

def init(manager):
    global g_manager
    g_manager = manager
    manager['utility'] = pluginmanager.ExtensionPoint()
    manager['utility.messageBox'] = messageBox
    manager['utility.askText'] = askText
    manager['utility.writeStatustext'] = writeStatustext
    manager['utility.getImage'] = getImage
    manager['utility.getBuiltinImage'] = getBuiltinImage
